@@ -1,3 +1,11 @@
+#########################################
+# Model.py                              #
+# Model                                 #
+#                                       #
+# @author Anirudhan Badrinath           #
+# Last edited: 07 April 2021            #
+#########################################
+
 import numpy as np
 import numbers
 import os
@@ -14,7 +22,7 @@ class Model:
     MODELS_BKT = ['multilearn', 'multiprior', 'multipair', 'multigs']
     MODEL_ARGS = ['parallel', 'num_fits', 'seed', 'defaults'] + MODELS_BKT
     FIT_ARGS = ['skills', 'num_fits', 'defaults',
-                            'parallel', 'forgets'] + MODELS_BKT
+                            'parallel', 'forgets', 'preload'] + MODELS_BKT
     CV_ARGS = FIT_ARGS + ['folds', 'seed']
     DEFAULTS = {'num_fits': 5,
                 'defaults': None,
@@ -31,7 +39,8 @@ class Model:
                     'multilearn': 'template_id',
                     'multiprior': 'correct',
                     'multipair': 'problem_id',
-                    'multigs': 'template_id'}
+                    'multigs': 'template_id',
+                    'folds': 'template_id'}
     INITIALIZABLE_PARAMS = ['prior', 'learns', 'guesses', 'slips', 'forgets']
 
     def __init__(self, **kwargs):
@@ -55,7 +64,7 @@ class Model:
         """
         Fits a BKT model given model and data information. Takes arguments skills,
         number of initialization fits, default column names (i.e. correct, skill_name),
-        parallelization, and model types.
+        parallelization, and model types. Resets model state if uninitialized.
 
         >>> model = Model(seed = 42)
         >>> model.fit(data_path = 'as.csv', forgets = True, skills = 'Box and Whisker')
@@ -63,16 +72,40 @@ class Model:
         0.6128265543747811
 
         """
+        if not self.manual_param_init:
+            self.fit_model = {}
+        self.partial_fit(data_path = data_path, data = data, **kwargs)
+
+    def partial_fit(self, data_path = None, data = None, **kwargs):
+        """
+        Partially fits a BKT model given model and data information. Takes arguments skills,
+        number of initialization fits, default column names (i.e. correct, skill_name),
+        parallelization, and model types. Behaviour is to ignore if the model is changed
+        between partial fits since parameters are copied but data is reprocessed. Note
+        that model type need not be specified when using partial fit after the first partial
+        fit.
+
+        >>> model = Model(seed = 42)
+        >>> model.partial_fit(data_path = 'as.csv', forgets = True, skills = 'Box and Whisker')
+        >>> model.partial_fit(data_path = 'as.csv', forgets = True, skills = 'Box and Whisker')
+        >>> model.partial_fit(data_path = 'as.csv', forgets = True, skills = 'Box and Whisker')
+        >>> model.evaluate(data_path = 'as.csv', metric = 'auc')
+        0.6579800168987382
+
+        """
+        self.manual_param_init = True
         self._check_data(data_path, data)
         self._check_args(Model.FIT_ARGS, kwargs)
         self._update_param(['skills', 'num_fits', 'defaults', 
                             'parallel', 'forgets'], kwargs)
-        self._update_param('model_type', self._update_defaults(kwargs))
-        if not self.manual_param_init:
+        if self.fit_model is None or self.fit_model == {}:
             self.fit_model = {}
+            self._update_param('model_type', self._update_defaults(kwargs))
         all_data = self._data_helper(data_path, data, self.defaults, self.skills, self.model_type)
+        self._update_param(['skills'], {'skills': list(all_data.keys())})
         for skill in all_data:
-            self.fit_model[skill] = self._fit(all_data[skill], skill, self.forgets)
+            self.fit_model[skill] = self._fit(all_data[skill], skill, self.forgets, 
+                                              preload = kwargs['preload'] if 'preload' in kwargs else False)
         self.manual_param_init = False
 
     def predict(self, data_path = None, data = None):
@@ -132,15 +165,24 @@ class Model:
 
         """
         self._check_data(data_path, data)
+        if not isinstance(metric, (tuple, list)):
+            metric = [metric]
         if self.fit_model is None:
             raise ValueError("model has not been fitted yet")
-        elif isinstance(metric, str):
-            if not metric in metrics.SUPPORTED_METRICS:
-                raise ValueError("metric must be one of: " + ", ".join(metrics.SUPPORTED_METRICS))
-            metric = getattr(metrics, metric)
+        else:
+            for i in range(len(metric)):
+                m = metric[i]
+                if isinstance(m, str):
+                    if not m in metrics.SUPPORTED_METRICS:
+                        raise ValueError("metric must be one of: " + ", ".join(metrics.SUPPORTED_METRICS))
+                    metric[i] = metrics.SUPPORTED_METRICS[m] 
+                elif not callable(m):
+                    raise ValueError("metric must either be a string, function or list/tuple of strings and functions")
+
         all_data = self._data_helper(data_path, data, self.defaults, self.skills, self.model_type,
                                      gs_ref = self.fit_model, resource_ref = self.fit_model)
-        return self._evaluate(all_data, metric)
+        results = self._evaluate(all_data, metric)
+        return results[0] if len(results) == 1 else results
 
     def crossvalidate(self, data = None, data_path = None, metric = metrics.rmse, **kwargs):
         """
@@ -167,12 +209,24 @@ class Model:
         [110 rows x 1 columns]
 
         """
+        metric_names = []
+        if not isinstance(metric, (tuple, list)):
+            metric = [metric]
         if not isinstance(data, pd.DataFrame) and not isinstance(data_path, str):
             raise ValueError("no data specified")
-        elif isinstance(metric, str):
-            if not metric in metrics.SUPPORTED_METRICS:
-                raise ValueError("metric must be one of: " + ", ".join(metrics.SUPPORTED_METRICS))
-            metric = getattr(metrics, metric)
+        else:
+            for i in range(len(metric)):
+                m = metric[i]
+                if isinstance(m, str):
+                    if not m in metrics.SUPPORTED_METRICS:
+                        raise ValueError("metric must be one of: " + ", ".join(metrics.SUPPORTED_METRICS))
+                    metric[i] = metrics.SUPPORTED_METRICS[m]
+                    metric_names.append(m)
+                elif callable(m):
+                    metric_names.append(m.__name__)
+                else:
+                    raise ValueError("metric must either be a string, function or list/tuple of strings and functions")
+
         self._check_args(Model.CV_ARGS, kwargs)
         self._update_param(['skills', 'num_fits', 'defaults', 
                             'parallel', 'forgets', 'seed', 'folds'], kwargs)
@@ -180,13 +234,17 @@ class Model:
         metric_vals = {}
         if not self.manual_param_init:
             self.fit_model = {}
-        all_data = self._data_helper(data_path, data, self.defaults, self.skills, self.model_type)
+        if isinstance(self.folds, str):
+            self._update_defaults({'folds': self.folds})
+        all_data = self._data_helper(data_path, data, self.defaults, self.skills, self.model_type, folds = isinstance(self.folds, str))
+        self._update_param(['skills'], {'skills': list(all_data.keys())})
         for skill in all_data:
             metric_vals[skill] = self._crossvalidate(all_data[skill], skill, metric)
         self.manual_param_init = False
         df = pd.DataFrame(metric_vals.items())
-        df.columns = ['skill', metric if isinstance(metric, str) else metric.__name__]
-        return df.set_index('skill')
+        df.columns = ['skill', 'dummy']
+        df[metric_names] = pd.DataFrame(df['dummy'].tolist(), index = df.index)
+        return df.set_index('skill').drop(columns = 'dummy')
 
     @property
     def coef_(self):
@@ -285,14 +343,14 @@ class Model:
         with open(os.path.normpath(loc + '/' + name), 'wb') as f:
             f.write(file_data.read())
 
-    def _data_helper(self, data_path, data, defaults, skills, model_type, gs_ref = None, resource_ref = None, return_df = False):
+    def _data_helper(self, data_path, data, defaults, skills, model_type, gs_ref = None, resource_ref = None, return_df = False, folds = False):
         """ Processes data given defaults, skills, and the model type. """
         if isinstance(data_path, str):
             data_p = data_helper.convert_data(data_path, skills, defaults = defaults, model_type = model_type, 
-                                              gs_refs = gs_ref, resource_refs = resource_ref, return_df = return_df)
+                                              gs_refs = gs_ref, resource_refs = resource_ref, return_df = return_df, folds = folds)
         elif isinstance(data, pd.DataFrame):
             data_p = data_helper.convert_data(data, skills, defaults = defaults, model_type = model_type,
-                                                gs_refs = gs_ref, resource_refs = resource_ref, return_df = return_df)
+                                                gs_refs = gs_ref, resource_refs = resource_ref, return_df = return_df, folds = folds)
         if not return_df:
             for d in data_p.values():
                 check_data.check_data(d)
@@ -301,7 +359,7 @@ class Model:
                 check_data.check_data(d)
         return data_p
 
-    def _fit(self, data, skill, forgets):
+    def _fit(self, data, skill, forgets, preload = False):
         """ Helper function for fitting data. """
         num_learns = len(data["resource_names"])
         num_gs = len(data["gs_names"])
@@ -312,15 +370,18 @@ class Model:
 
         for i in range(num_fit_initializations):
             fitmodel = random_model_uni.random_model_uni(num_learns, num_gs)
-            if self.manual_param_init and skill in self.fit_model:
-                for var in self.fit_model[skill]:
-                    fitmodel[var] = self.fit_model[skill][var]
-
             if forgets:
                 fitmodel["forgets"] = np.random.uniform(size = fitmodel["forgets"].shape)
-            fitmodel, log_likelihoods = EM_fit.EM_fit(fitmodel, data, parallel = self.parallel)
-            if log_likelihoods[-1] > best_likelihood:
-                best_likelihood = log_likelihoods[-1]
+            if self.manual_param_init and skill in self.fit_model:
+                for var in self.fit_model[skill]:
+                    if var in fitmodel:
+                        fitmodel[var] = self.fit_model[skill][var]
+            if not preload:
+                fitmodel, log_likelihoods = EM_fit.EM_fit(fitmodel, data, parallel = self.parallel)
+                if log_likelihoods[-1] > best_likelihood:
+                    best_likelihood = log_likelihoods[-1]
+                    best_model = fitmodel
+            else:
                 best_model = fitmodel
         fit_model = best_model
         fit_model["learns"] = fit_model["As"][:, 1, 0]
@@ -344,18 +405,25 @@ class Model:
             true = np.append(true, real_data.sum(axis = 0))
             pred = np.append(pred, correct_predictions)
         true = true - 1
-        return metric(true, pred)
+        try:
+            res = [m(true, pred) for m in metric]
+        except ValueError:
+            res = [m(true, pred.round(0)) for m in metric]
+        return res
 
     def _crossvalidate(self, data, skill, metric):
         """ Helper function for crossvalidating. """
-        return crossvalidate.crossvalidate(self, data, skill, self.folds, metric, self.seed)
+        if isinstance(self.folds, str):
+            return crossvalidate.crossvalidate(self, data, skill, self.folds, metric, self.seed, True)
+        else:
+            return crossvalidate.crossvalidate(self, data, skill, self.folds, metric, self.seed)
 
     def _format_param(self, skill, param, value):
         """ Formats parameter for nice printing. """
         if isinstance(value, np.ndarray):
             ptype = 'resource_names' if (param == 'learns' or param == 'forgets') \
                                      else 'gs_names'
-            return dict(zip(self.fit_model[skill][ptype], value))
+            return dict(sorted(zip(self.fit_model[skill][ptype], value)))
         else:
             return {'default': value}
 
