@@ -1,3 +1,11 @@
+#########################################
+# data_helper.py                        #
+# data_helper                           #
+#                                       #
+# @author Frederic Wang                 #
+# Last edited: 07 April 2021            #
+#########################################
+
 import sys
 sys.path.append('../')
 import os
@@ -6,7 +14,7 @@ import numpy as np
 import io
 import requests
 
-def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None, resource_refs=None, return_df = False):
+def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None, resource_refs=None, return_df = False, folds=False):
     if model_type:
         multilearn, multiprior, multipair, multigs = model_type
     else:
@@ -51,6 +59,7 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
                  'multiprior': 'correct',
                  'multipair': 'template_id',
                  'multigs': 'template_id',
+                 'folds': 'user_id',
                  }
 
     # default column names for cognitive tutors
@@ -62,6 +71,7 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
                 'multiprior': 'Correct First Attempt',
                 'multipair': 'Problem Name',
                 'multigs': 'Problem Name',
+                'folds': 'Anon Student Id',
                                  }
 
     # integrate custom defaults with default assistments/ct columns if they are still unspecified
@@ -101,20 +111,22 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
     
     if "original" in df.columns:
         df = df[(df["original"]==1)]
+        
+    df[defaults["skill_name"]] = df[defaults["skill_name"]].apply(str)
     
     datas = {}
     skill_name = '^(' + skill_name + ')$'
-    all_skills = pd.Series(df[defaults["skill_name"]].unique()).dropna().apply(lambda x: str(x))
+    all_skills = pd.Series(df[defaults["skill_name"]].unique()).dropna()
     all_skills = all_skills[all_skills.str.match(skill_name).astype(bool)]
     if all_skills.empty:
         raise ValueError("no matching skills")
     for skill_ in all_skills:
 
-        if resource_refs is None:
+        if resource_refs is None or skill_ not in resource_refs:
             resource_ref = None
         else:
             resource_ref = resource_refs[skill_]["resource_names"]
-        if gs_refs is None:
+        if gs_refs is None or skill_ not in gs_refs:
             gs_ref = None
         else:
             gs_ref = gs_refs[skill_]["gs_names"]
@@ -127,6 +139,8 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
         stored_index = df3.index.copy()
 
         # convert from 0=incorrect,1=correct to 1=incorrect,2=correct
+        if set(df3.loc[:,defaults["correct"]].unique()) - set([-1, 0, 1]) != set():
+            raise ValueError("correctness must be -1 (no response), 0 (incorrect), or 1 (correct)")
         df3.loc[:,defaults["correct"]]+=1
         
         # array representing correctness of student answers
@@ -154,7 +168,7 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
             resources = np.ones(len(data), dtype=np.int64)
             if resource_ref is None:
                 new_resource_ref = {}
-                new_resource_ref["N/A"] = 1 #no pair
+                new_resource_ref["Default"] = 1 #no pair
             for i in range(len(df3)):
                 # for the first entry of a new student, no pair
                 if i == 0 or df3[i:i+1][defaults["user_id"]].values != df3[i-1:i][defaults["user_id"]].values:
@@ -182,7 +196,7 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
             all_priors = df3[defaults["multiprior"]].unique()
             if resource_ref is None:
                 resource_ref = dict(zip(all_priors,range(2, len(df3[defaults["multiprior"]].unique())+2)))
-                resource_ref["N/A"] = 1
+                resource_ref["Default"] = 1
             else:
                 for i in all_priors:
                     if i not in resource_ref:
@@ -256,6 +270,8 @@ def convert_data(url, skill_name, defaults=None, model_type=None, gs_refs=None,
         Data["resource_names"]=resource_ref
         Data["gs_names"]=gs_ref
         Data["index"]=stored_index
+        if folds:
+            Data["folds"] = np.array(df3[defaults["folds"]])
 
         datas[skill_] = Data
 
